/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
**	Header file used for the node tracking routines.
*/



enum	psit {
	okay,
	thinking,
	conflict
};

struct	prop {
	char	*name;
	int	mark;
	struct	prop	*next;
};

struct	jobinfo {
	job	*job;
	struct	jobinfo	*next;
};

struct	pbsnode {
	struct	prop	*first;
	struct	prop	*last;
	int		nprops;
	int		stream;
	u_long		*addrs;
	struct	jobinfo	*jobs;
	resource_t	allocto;
	enum	psit	flag;
	unsigned short	inuse:12;
	unsigned short	ntype:4;
	short		order;
};

#define	INUSE_FREE	 0x00	/* Node is available			*/
#define	INUSE_OFFLINE	 0x01	/* Node was removed by administrator	*/
#define	INUSE_DOWN	 0x02	/* Node is down/unresponsive 		*/
#define	INUSE_RESERVE	 0x04	/* Node being reserved by scheduler	*/
#define	INUSE_JOB	 0x08	/* Node in use by job (exclusive use)	*/
#define INUSE_JOBSHARE	 0x10	/* Node is use by job(s) (time shared)	*/
#define INUSE_UNKNOWN	 0x20	/* Node has not been heard from yet	*/
#define INUSE_DELETED	 0x80	/* Node is "deleted"			*/
#define INUSE_NEEDS_HELLO_PING	 0x100	/*node needs to be informed of a*/
					/*new qmgr created node         */

#define NTYPE_CLUSTER	 0x00	/* Node is normal allocatable node	*/
#define NTYPE_TIMESHARED 0x01	/* Node is Time Shared Node		*/

#define TIMESHARED_SUFFIX	"ts"
#define PBS_MAXNODENAME	80	/* upper bound on the node name size    */

#define	PBSNODE_STATE		0x1		 /* characteristic code */
#define PBSNODE_PROPERTIES	0x2		 /* characteristic code */
#define PBSNODE_NTYPE		0x3		 /* characteristic code */
#define PBSNODE_STATE_MASK	0x3f		 /* relevant state bits */
#define PBSNODE_NTYPE_MASK	0xf		 /* relevant ntype bits */

#define WRITENODE_STATE		0x1		 /*associated w/ offline*/
#define WRITE_NEW_NODESFILE	0x2 /*changed: deleted,ntype,or properties*/

/*
 * Although at the present time a struct pbsnode doesn't have an array of
 * attributes associated with it, it is convenient for the server's req_manager
 * function to use as much of the existing attribute processing functions and
 * philosophy as practical to carry out any node modification requests sent via
 * "qmgr".   For this reason, and for the fact that some day struct pbsnodes
 * may be redefined to possess attributes, it is convenient to pretend that
 * nodes have an associated attributes array just as a server or a queue or
 * a job would have.
 *
 * The following enum defines the index into the node_attr_def[]  array.
 */

enum nodeattr {
	ND_ATR_state,
	ND_ATR_properties,
	ND_ATR_ntype,
	ND_ATR_jobs,
	ND_ATR_LAST	/* WARNING: Must be the highest valued enum */
};

extern attribute_def node_attr_def[];
extern	struct pbsnode	**pbsnlist;
extern	int		svr_totnodes;
extern	struct tree_t	*ipaddrs;
extern	struct tree_t	*streams;

extern	int	update_nodes_file A_(());

#ifdef BATCH_REQUEST_H
extern	void	initialize_pbsnode A_((struct pbsnode*, struct prop*, u_long*, int));
extern	void	effective_node_delete A_((struct pbsnode*));
extern	void	setup_notification A_((void));
extern  struct pbsnode  *find_nodebyname A_((char *));
extern	void	free_prop_list A_((struct prop*));
extern	void	recompute_ntype_cnts A_(());
extern	int	process_host_name_part A_((struct batch_request*, u_long**, struct prop**, int*));
#endif	/* BATCH_REQUEST_H */
